
# q-expansion of newform 3536.2.a.r, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3536
weight = 2
poly_data = [-5, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, -1], [-1, 0], [2, 1], [-1, -1], [-1, 0], [1, 0], [-3, 1], [-4, 2], [9, 0], [-3, -2], [-3, -2], [0, 0], [-9, 0], [0, 2], [-6, 1], [-5, 2], [10, -1], [8, 2], [-2, 0], [5, -2], [5, -2], [-1, 2], [-7, 5], [6, -5], [-1, -4], [1, 0], [-1, 3], [-8, 5], [-2, 1], [-3, -5], [-7, 2], [1, 1], [-4, -6], [-5, -5], [-8, 6], [-9, -2], [-14, 1], [-3, -8], [1, -4], [7, 4], [-18, 1], [1, 6], [4, 2], [12, -6], [-11, 0], [-15, -2], [-10, -1], [4, -2], [7, -2], [1, -10], [15, -6], [7, -6], [2, 3], [-3, 4], [2, 0], [-15, 3], [-22, -3], [-18, 4], [-4, -9], [-8, 1], [26, -3], [-17, -1], [-17, -2], [-1, 0], [17, 0], [11, 0], [-9, 1], [5, -6], [19, -3], [0, 8], [14, 1], [16, -5], [2, 9], [2, 6], [-1, -10], [-15, 5], [-6, -4], [-5, -5], [6, -4], [19, -3], [-9, 1], [32, -4], [2, -4], [10, 3], [5, -2], [10, -9], [-6, 11], [11, 6], [1, -6], [-2, -1], [-10, -9], [23, 0], [-1, 5], [2, 4], [-23, -3], [-23, 2], [19, -2], [-17, 15], [-1, 1], [-6, 15], [20, -2], [-26, -5], [-13, -3], [-24, -4], [-6, 13], [8, 5], [-20, 3], [-5, 0], [12, -17], [-7, 2], [-27, -6], [15, 2], [-41, 4], [-25, 4], [-3, 2], [2, -11], [28, -1], [3, 14], [17, -10], [22, 6], [4, 10], [-4, -15], [14, -10], [-32, 10], [-13, -5], [-11, -7], [6, 4], [-10, -8], [11, 4], [18, -11], [-6, -6], [8, -8], [-3, 3], [25, -10], [-38, -1], [7, 1], [19, -18], [3, 15], [-12, -8], [-19, -4], [-15, 22], [-13, 2], [10, -17], [12, -2], [11, -1], [-33, -4], [11, -14], [1, -7], [39, 0], [19, 3], [-21, 7], [19, 0], [37, 5], [-31, 3], [35, -4], [40, -2], [-13, 3], [-17, 6], [-27, -1], [4, 0], [22, 2], [39, 2], [-40, 5], [20, 5], [-11, 14], [5, 1], [-3, -14], [-9, -13], [-24, -3], [-37, 8], [-24, -1], [-5, 4], [25, 2], [19, -4], [19, -17], [-5, -15], [-28, 17], [23, -4], [23, 8], [43, -1], [23, -4], [6, 6], [-14, -9], [16, 14], [-21, 8], [-13, 15], [28, -15], [-7, 6], [-1, -13], [23, -6], [27, 12], [-54, 4], [14, -9], [-17, 3], [-7, 8], [-8, -9], [17, -7], [40, -3], [3, -18], [4, 16], [-30, 18], [-27, 17], [30, -3], [-28, 14], [-38, 7], [-17, 8], [-29, 17], [-5, -14], [-23, 3], [7, 6], [-15, -12], [-29, 12], [2, 19], [1, 4], [2, 8], [-12, 21], [-49, 1], [23, 11], [3, 5], [-41, 11], [27, 2], [6, 1], [31, -5], [7, 21], [10, -9], [9, -13], [-8, -8], [-5, 5], [42, 1], [16, 13], [24, 14], [26, -14], [56, -4], [26, -3], [7, 6], [-18, 6], [-57, 1], [45, -10], [-59, -1], [-54, 7], [-28, 2], [-25, 21], [49, -8], [-12, 24], [-65, 6], [40, -3], [-16, 24], [-28, 18], [3, -24], [-38, 3], [-51, 9], [-29, -1], [-22, 2], [-2, -2], [-1, 17], [-24, 9], [15, -10], [37, 10], [-19, 22], [26, 1], [4, -12], [29, -1], [-41, 12], [2, -17], [31, 3], [1, -26], [3, -6], [7, 3], [-18, -10], [-15, 21], [-16, 22], [5, -2], [-47, -10], [5, 25], [-4, -4], [25, 7], [-38, 2], [26, -4], [24, -4], [-4, 21], [1, -5], [27, 16], [-17, 6], [5, -15], [-22, 17], [1, -27], [55, -15], [17, 14], [-65, 4], [-21, 24], [-4, -28], [-41, 4], [14, -28], [-19, 0], [-12, -24], [55, -3], [42, 0], [3, 24], [-23, 4], [-12, -6], [16, -26], [25, -25], [-16, 7], [-16, -9], [-12, 14], [8, 9], [-10, 20], [5, 26], [43, -3], [-59, -6], [0, -4], [-15, 15], [-24, 10], [-50, 19], [-14, 36], [22, -16], [-28, 11], [7, 7], [4, -18], [-7, 13], [-22, 29], [-4, 15], [-49, -4], [-26, 20], [38, -8], [4, 14], [-48, 0], [-64, -4], [12, 15], [41, 8], [-32, -10], [-35, 7], [-5, 9], [-72, -6], [65, -11], [-12, 0], [-26, 1], [41, -5], [58, -16], [6, 5], [73, -2], [38, 8], [-65, 0], [3, 14], [30, -15], [-11, 9], [27, -4], [34, 2], [-33, -20], [9, -23], [41, 5], [-20, -12], [-32, 14], [14, -1], [-45, 6], [9, 12], [-6, 4], [36, 4], [-66, 9], [17, 19], [23, -13], [3, -2], [-81, 2], [-1, -27], [-44, 3], [24, -27], [-11, -16], [-20, 2], [52, -9], [-75, 12], [9, -3], [-29, -8], [-30, -11], [7, -15], [-1, 15], [-2, -21], [-2, -9], [-66, 11], [-6, -16], [55, -18], [-5, 0], [-35, 3], [20, 5], [20, -5], [24, 0], [0, -10], [-25, -12], [42, -6], [40, -17], [-33, 28], [-41, 17], [1, 2], [43, 14], [-25, 2], [25, 9], [11, -9], [3, 22], [40, 14], [37, 13], [-41, -4], [72, -12], [-37, 19], [-6, 3], [-34, 20], [12, -36], [22, -13], [-44, 15], [12, -18], [-86, -1], [-33, 29], [-52, 12], [11, -5], [35, -27], [13, 6], [75, 9], [-19, -10], [-83, -3], [-68, 10], [0, 2], [12, -14], [-39, 20], [-58, 14], [9, 3], [-1, 23], [57, 6], [32, 7], [-9, -20], [-2, 10], [-46, 7]]
